"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const mixed_1 = require("../helpers/mixed");
const download_1 = require("./download");
async function postinstall() {
    if ("NO_ELM_TOOLING_POSTINSTALL" in process.env) {
        return 0;
    }
    const result = await download_1.default();
    switch (result.tag) {
        case "Exit":
            return result.statusCode;
        case "Success":
            return linkTools(result.tools);
    }
}
exports.default = postinstall;
function linkTools(tools) {
    const nodeModulesPath = mixed_1.findClosest("node_modules");
    if (nodeModulesPath === undefined) {
        console.error("No node_modules/ folder found. Install your npm dependencies before running this script.");
        return 1;
    }
    const nodeModulesBinPath = path.join(nodeModulesPath, ".bin");
    try {
        fs.mkdirSync(nodeModulesBinPath, { recursive: true });
    }
    catch (errorAny) {
        const error = errorAny;
        console.error(`Failed to create ${nodeModulesBinPath}:\n${error.message}`);
    }
    for (const tool of tools) {
        const linkPath = path.join(nodeModulesBinPath, tool.name);
        // Just like npm, just overwrite whatever links are already in
        // `node_modules/.bin/`. Most likely it’s either old links from for example
        // the `elm` npm package, or links from previous runs of this script.
        try {
            fs.unlinkSync(linkPath);
        }
        catch (errorAny) {
            const error = errorAny;
            if (error.code !== "ENOENT") {
                console.error(`Failed to remove old link for ${tool.name} at ${linkPath}:\n${error.message}`);
                return 1;
            }
        }
        try {
            fs.symlinkSync(tool.absolutePath, linkPath);
        }
        catch (errorAny) {
            const error = errorAny;
            console.error(`Failed to create link for ${tool.name} at ${linkPath}:\n${error.message}`);
            return 1;
        }
        console.log(`${mixed_1.bold(`${tool.name} ${tool.version}`)} link created: ${mixed_1.dim(`${linkPath} -> ${tool.absolutePath}`)}`);
    }
    return 0;
}
